#!/usr/bin/env python

"""
    MC6809 - 6809 CPU emulator in Python
    =======================================

    some code is borrowed from:
    XRoar emulator by Ciaran Anscomb (GPL license) more info, see README

    :copyleft: 2013-2014 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


from MC6809.utils.humanize import cc_value2txt
import logging

log=logging.getLogger("MC6809")


class ValueStorage(object):
    def __init__(self, name, initial_value):
        self.name = name
        self.value = initial_value

    def set(self, v):
        self.value = v
        return self.value # e.g.: r = operand.set(a + 1)
    def get(self):
        return self.value

    # FIXME:
    def decrement(self, value=1):
        return self.set(self.value - value)
    def increment(self, value=1):
        return self.set(self.value + value)

    def __str__(self):
        return "<%s:$%x>" % (self.name, self.value)
    __repr__ = __str__


class UndefinedRegister(ValueStorage):
    # used in TFR and EXG
    WIDTH = 16 # 16 Bit
    name = "undefined!"
    value = 0xffff
    def __init__(self):
        pass
    def set(self, v):
        log.warning("Set value to 'undefined' register!")
        pass

    def get(self):
        return 0xffff


class ValueStorage8Bit(ValueStorage):
    WIDTH = 8 # 8 Bit

    def set(self, v):
        if v > 0xff:
#            log.info(" **** Value $%x is to big for %s (8-bit)", v, self.name)
            v = v & 0xff
#            log.info(" ^^^^ Value %s (8-bit) wrap around to $%x", self.name, v)
        elif v < 0:
#            log.info(" **** %s value $%x is negative", self.name, v)
            v = 0x100 + v
#            log.info(" **** Value %s (8-bit) wrap around to $%x", self.name, v)
        self.value = v
        return self.value # e.g.: r = operand.set(a + 1)
    def __str__(self):
        return "%s=%02x" % (self.name, self.value)


class ValueStorage16Bit(ValueStorage):
    WIDTH = 16 # 16 Bit

    def set(self, v):
        if v > 0xffff:
#            log.info(" **** Value $%x is to big for %s (16-bit)", v, self.name)
            v = v & 0xffff
#            log.info(" ^^^^ Value %s (16-bit) wrap around to $%x", self.name, v)
        elif v < 0:
#            log.info(" **** %s value $%x is negative", self.name, v)
            v = 0x10000 + v
#            log.info(" **** Value %s (16-bit) wrap around to $%x", self.name, v)
        self.value = v
        return self.value # e.g.: r = operand.set(a + 1)
    def __str__(self):
        return "%s=%04x" % (self.name, self.value)



def _register_bit(key):
    def set_flag(self, value):
        assert value in (0, 1)
        self._register[key] = value
    def get_flag(self):
        return self._register[key]
    return property(get_flag, set_flag)


class ConditionCodeRegister(object):
    """ CC - 8 bit condition code register bits """

    WIDTH = 8 # 8 Bit

    def __init__(self, *cmd_args, **kwargs):
        self.name = "CC"
        self._register = {}
        self.set(0x0) # create all keys in dict with value 0

    E = _register_bit("E") # E - 0x80 - bit 7 - Entire register state stacked
    F = _register_bit("F") # F - 0x40 - bit 6 - FIRQ interrupt masked
    H = _register_bit("H") # H - 0x20 - bit 5 - Half-Carry
    I = _register_bit("I") # I - 0x10 - bit 4 - IRQ interrupt masked
    N = _register_bit("N") # N - 0x08 - bit 3 - Negative result (twos complement)
    Z = _register_bit("Z") # Z - 0x04 - bit 2 - Zero result
    V = _register_bit("V") # V - 0x02 - bit 1 - Overflow
    C = _register_bit("C") # C - 0x01 - bit 0 - Carry (or borrow)

    ####

    def set(self, status):
        self.E, self.F, self.H, self.I, self.N, self.Z, self.V, self.C = \
            [0 if status & x == 0 else 1 for x in (128, 64, 32, 16, 8, 4, 2, 1)]

    def get(self):
        return self.C | \
            self.V << 1 | \
            self.Z << 2 | \
            self.N << 3 | \
            self.I << 4 | \
            self.H << 5 | \
            self.F << 6 | \
            self.E << 7

    @property
    def get_info(self):
        """
        >>> cc=ConditionCodeRegister()
        >>> cc.set(0xa1)
        >>> cc.get_info
        'E.H....C'
        """
        return cc_value2txt(self.get())

    def __str__(self):
        return "%s=%s" % (self.name, self.get_info)

    ####

    """
    #define SET_Z(r)          ( REG_CC |= ((r) ? 0 : CC_Z) )
    #define SET_N8(r)         ( REG_CC |= (r&0x80)>>4 )
    #define SET_N16(r)        ( REG_CC |= (r&0x8000)>>12 )
    #define SET_H(a,b,r)      ( REG_CC |= ((a^b^r)&0x10)<<1 )
    #define SET_C8(r)         ( REG_CC |= (r&0x100)>>8 )
    #define SET_C16(r)        ( REG_CC |= (r&0x10000)>>16 )
    #define SET_V8(a,b,r)     ( REG_CC |= ((a^b^r^(r>>1))&0x80)>>6 )
    #define SET_V16(a,b,r)    ( REG_CC |= ((a^b^r^(r>>1))&0x8000)>>14 )
    """

    def set_H(self, a, b, r):
        if self.H == 0:
            r2 = (a ^ b ^ r) & 0x10
            self.H = 0 if r2 == 0 else 1
#            log.debug("\tset_H(): set half-carry flag to %i: ($%02x ^ $%02x ^ $%02x) & 0x10 = $%02x",
#                self.H, a, b, r, r2
#            )
#        else:
#            log.debug("\rset_H(): leave old value 1")

    def set_Z8(self, r):
        if self.Z == 0:
            r2 = r & 0xff
            self.Z = 1 if r2 == 0 else 0
#            log.debug("\tset_Z8(): set zero flag to %i: $%02x & 0xff = $%02x",
#                self.Z, r, r2
#            )
#        else:
#            log.debug("\tset_Z8(): leave old value 1")

    def set_Z16(self, r):
        if self.Z == 0:
            r2 = r & 0xffff
            self.Z = 1 if r2 == 0 else 0
#            log.debug("\tset_Z16(): set zero flag to %i: $%04x & 0xffff = $%04x",
#                self.Z, r, r2
#            )
#        else:
#            log.debug("\tset_Z16(): leave old value 1")

    def set_N8(self, r):
        if self.N == 0:
            r2 = r & 0x80
            self.N = 0 if r2 == 0 else 1
#            log.debug("\tset_N8(): set negative flag to %i: ($%02x & 0x80) = $%02x",
#                self.N, r, r2
#            )
#        else:
#            log.debug("\tset_N8(): leave old value 1")

    def set_N16(self, r):
        if self.N == 0:
            r2 = r & 0x8000
            self.N = 0 if r2 == 0 else 1
#            log.debug("\tset_N16(): set negative flag to %i: ($%04x & 0x8000) = $%04x",
#                self.N, r, r2
#            )
#        else:
#            log.debug("\tset_N16(): leave old value 1")

    def set_C8(self, r):
        if self.C == 0:
            r2 = r & 0x100
            self.C = 0 if r2 == 0 else 1
#            log.debug("\tset_C8(): carry flag to %i: ($%02x & 0x100) = $%02x",
#                self.C, r, r2
#            )
#         else:
#            log.debug("\tset_C8(): leave old value 1")

    def set_C16(self, r):
        if self.C == 0:
            r2 = r & 0x10000
            self.C = 0 if r2 == 0 else 1
#            log.debug("\tset_C16(): carry flag to %i: ($%04x & 0x10000) = $%04x",
#                self.C, r, r2
#            )
#         else:
#            log.debug("\tset_C16(): leave old value 1")

    def set_V8(self, a, b, r):
        if self.V == 0:
            r2 = (a ^ b ^ r ^ (r >> 1)) & 0x80
            self.V = 0 if r2 == 0 else 1
#            log.debug("\tset_V8(): overflow flag to %i: (($%02x ^ $%02x ^ $%02x ^ ($%02x >> 1)) & 0x80) = $%02x",
#                self.V, a, b, r, r, r2
#            )
#         else:
#            log.debug("\tset_V8(): leave old value 1")

    def set_V16(self, a, b, r):
        if self.V == 0:
            r2 = (a ^ b ^ r ^ (r >> 1)) & 0x8000
            self.V = 0 if r2 == 0 else 1
#            log.debug("\tset_V16(): overflow flag to %i: (($%04x ^ $%04x ^ $%04x ^ ($%04x >> 1)) & 0x8000) = $%04x",
#                self.V, a, b, r, r, r2
#            )
#         else:
#            log.debug("\tset_V16(): leave old value 1")

    ####

    def clear_NZ(self):
#        log.debug("\tclear_NZ()")
        self.N = 0
        self.Z = 0

    def clear_NZC(self):
#        log.debug("\tclear_NZC()")
        self.N = 0
        self.Z = 0
        self.C = 0

    def clear_NZV(self):
#        log.debug("\tclear_NZV()")
        self.N = 0
        self.Z = 0
        self.V = 0

    def clear_NZVC(self):
#        log.debug("\tclear_NZVC()")
        self.N = 0
        self.Z = 0
        self.V = 0
        self.C = 0

    def clear_HNZVC(self):
#        log.debug("\tclear_HNZVC()")
        self.H = 0
        self.N = 0
        self.Z = 0
        self.V = 0
        self.C = 0

    ####

    def update_NZ_8(self, r):
        self.set_N8(r)
        self.set_Z8(r)

    def update_0100(self):
        """ CC bits "HNZVC": -0100 """
        self.N = 0
        self.Z = 1
        self.V = 0
        self.C = 0

    def update_NZ01_8(self, r):
        self.set_N8(r)
        self.set_Z8(r)
        self.V = 0
        self.C = 1

    def update_NZ_16(self, r):
        self.set_N16(r)
        self.set_Z16(r)

    def update_NZ0_8(self, r):
        self.set_N8(r)
        self.set_Z8(r)
        self.V = 0

    def update_NZ0_16(self, r):
        self.set_N16(r)
        self.set_Z16(r)
        self.V = 0

    def update_NZC_8(self, r):
        self.set_N8(r)
        self.set_Z8(r)
        self.set_C8(r)

    def update_NZVC_8(self, a, b, r):
        self.set_N8(r)
        self.set_Z8(r)
        self.set_V8(a, b, r)
        self.set_C8(r)

    def update_NZVC_16(self, a, b, r):
        self.set_N16(r)
        self.set_Z16(r)
        self.set_V16(a, b, r)
        self.set_C16(r)

    def update_HNZVC_8(self, a, b, r):
        self.set_H(a, b, r)
        self.set_N8(r)
        self.set_Z8(r)
        self.set_V8(a, b, r)
        self.set_C8(r)


class ConcatenatedAccumulator(object):
    """
    6809 has register D - 16 bit concatenated reg. (A + B)
    """
    WIDTH = 16 # 16 Bit

    def __init__(self, name, a, b):
        self.name = name
        self._a = a
        self._b = b

    def set(self, value):
        self._a.set(value >> 8)
        self._b.set(value & 0xff)

    def get(self):
        a = self._a.get()
        b = self._b.get()
        return a * 256 + b

    def __str__(self):
        return "%s=%04x" % (self.name, self.get())


if __name__ == "__main__":
    import doctest
    print(doctest.testmod())



#!/usr/bin/env python

"""
    6809 unittests
    ~~~~~~~~~~~~~~

    :created: 2013 by Jens Diemer - www.jensdiemer.de
    :copyleft: 2013-2014 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from __future__ import absolute_import, division, print_function

import hashlib
import logging
import unittest

try:
    import queue # Python 3
except ImportError:
    import Queue as queue # Python 2

from MC6809.utils.byte_word_values import bin2hexline
from MC6809.components.cpu6809 import CPU
from MC6809.components.memory import Memory
from MC6809.components.cpu_utils.MC6809_registers import ConditionCodeRegister, ValueStorage8Bit
from MC6809.tests.test_config import TestCfg


log = logging.getLogger("MC6809")


class BaseTestCase(unittest.TestCase):
    """
    Only some special assertments.
    """
    maxDiff=3000

    def assertHexList(self, first, second, msg=None):
        first = ["$%x" % value for value in first]
        second = ["$%x" % value for value in second]
        self.assertEqual(first, second, msg)

    def assertEqualHex(self, hex1, hex2, msg=None):
        first = "$%x" % hex1
        second = "$%x" % hex2
        if msg is None:
            msg = "%s != %s" % (first, second)
        self.assertEqual(first, second, msg)

    def assertIsByteRange(self, value):
        self.assertTrue(0x0 <= value, "Value (dez: %i - hex: %x) is negative!" % (value, value))
        self.assertTrue(0xff >= value, "Value (dez: %i - hex: %x) is greater than 0xff!" % (value, value))

    def assertIsWordRange(self, value):
        self.assertTrue(0x0 <= value, "Value (dez: %i - hex: %x) is negative!" % (value, value))
        self.assertTrue(0xffff >= value, "Value (dez: %i - hex: %x) is greater than 0xffff!" % (value, value))

    def assertEqualHexByte(self, hex1, hex2, msg=None):
        self.assertIsByteRange(hex1)
        self.assertIsByteRange(hex2)
        first = "$%02x" % hex1
        second = "$%02x" % hex2
        if msg is None:
            msg = "%s != %s" % (first, second)
        self.assertEqual(first, second, msg)

    def assertEqualHexWord(self, hex1, hex2, msg=None):
        self.assertIsWordRange(hex1)
        self.assertIsWordRange(hex2)
        first = "$%04x" % hex1
        second = "$%04x" % hex2
        if msg is None:
            msg = "%s != %s" % (first, second)
        self.assertEqual(first, second, msg)

    def assertBinEqual(self, bin1, bin2, msg=None, width=16):
        first = bin2hexline(bin1, width=width)
        second = bin2hexline(bin2, width=width)
        self.assertSequenceEqual(first, second, msg)

        # first = "\n".join(bin2hexline(bin1, width=width))
        # second = "\n".join(bin2hexline(bin2, width=width))
        # self.assertMultiLineEqual(first, second, msg)


class BaseCPUTestCase(BaseTestCase):
    UNITTEST_CFG_DICT = {
        "verbosity":None,
        "display_cycle":False,
        "trace":None,
        "bus_socket_host":None,
        "bus_socket_port":None,
        "ram":None,
        "rom":None,
        "max_ops":None,
        "use_bus":False,
    }
    def setUp(self):
        cfg = TestCfg(self.UNITTEST_CFG_DICT)
        memory = Memory(cfg)
        self.cpu = CPU(memory, cfg)

    def cpu_test_run(self, start, end, mem):
        for cell in mem:
            self.assertLess(-1, cell, "$%x < 0" % cell)
            self.assertGreater(0x100, cell, "$%x > 0xff" % cell)
        log.debug("memory load at $%x: %s", start,
            ", ".join(["$%x" % i for i in mem])
        )
        self.cpu.memory.load(start, mem)
        if end is None:
            end = start + len(mem)
        self.cpu.test_run(start, end)
    cpu_test_run.__test__=False # Exclude from nose

    def cpu_test_run2(self, start, count, mem):
        for cell in mem:
            self.assertLess(-1, cell, "$%x < 0" % cell)
            self.assertGreater(0x100, cell, "$%x > 0xff" % cell)
        self.cpu.memory.load(start, mem)
        self.cpu.test_run2(start, count)
    cpu_test_run2.__test__=False # Exclude from nose

    def assertMemory(self, start, mem):
        for index, should_byte in enumerate(mem):
            address = start + index
            is_byte = self.cpu.memory.read_byte(address)

            msg = "$%02x is not $%02x at address $%04x (index: %i)" % (
                is_byte, should_byte, address, index
            )
            self.assertEqual(is_byte, should_byte, msg)


class BaseStackTestCase(BaseCPUTestCase):
    INITIAL_SYSTEM_STACK_ADDR = 0x1000
    INITIAL_USER_STACK_ADDR = 0x2000
    def setUp(self):
        super(BaseStackTestCase, self).setUp()
        self.cpu.system_stack_pointer.set(self.INITIAL_SYSTEM_STACK_ADDR)
        self.cpu.user_stack_pointer.set(self.INITIAL_USER_STACK_ADDR)


class TestCPU(object):
    def __init__(self):
        self.accu_a = ValueStorage8Bit("A", 0) # A - 8 bit accumulator
        self.accu_b = ValueStorage8Bit("B", 0) # B - 8 bit accumulator
        # 8 bit condition code register bits: E F H I N Z V C
        self.cc = ConditionCodeRegister()



def print_cpu_state_data(state):
    print("cpu state data %r (ID:%i):" % (state.__class__.__name__, id(state)))
    for k, v in sorted(state.items()):
        if k == "RAM":
            # v = ",".join(["$%x" % i for i in v])
            print("\tSHA from RAM:", hashlib.sha224(repr(v)).hexdigest())
            continue
        if isinstance(v, int):
            v = "$%x" % v
        print("\t%r: %s" % (k, v))




#!/usr/bin/env python

"""
    DragonPy - Dragon 32 emulator in Python
    =======================================

    :copyleft: 2013-2015 by the DragonPy team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import os
import subprocess
import sys
import unittest

CLI = os.path.normpath(
    os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "..", "cli.py"
    )
)


class CLITestCase(unittest.TestCase):
    """
    TODO: Do more than this simple tests
    """

    def _get(self, *args):
        self.assertTrue(os.path.isfile(CLI), "Not a file: %r" % CLI)
        cmd_args = [
            sys.executable,
            CLI
        ]
        cmd_args += args
        # print("\nStartup CLI with: %s" % " ".join(cmd_args[1:]))

        p = subprocess.Popen(
            cmd_args,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )
        retcode = p.wait()

        cli_out = p.stdout.read()
        p.stdout.close()
        cli_err = p.stderr.read()
        p.stderr.close()

        if retcode != 0:
            msg = (
                "subprocess returned %s.\n"
                " *** stdout: ***\n"
                "%s\n"
                " *** stderr: ***\n"
                "%s\n"
                "****************\n"
            ) % (retcode, cli_out, cli_err)
            self.assertEqual(retcode, 0, msg=msg)

        return cli_out, cli_err

    def assertInMultiline(self, members, container):
        for member in members:
            msg = "%r not found in:\n%s" % (member, container)
            # self.assertIn(member, container, msg) # Bad error message :(
            if not member in container:
                self.fail(msg)

    def assertNotInMultiline(self, members, container):
        for member in members:
            if member in container:
                self.fail("%r found in:\n%s" % (member, container))

    def test_main_help(self):
        cli_out, cli_err = self._get("--help")
        self.assertInMultiline([
            "cli.py [OPTIONS] COMMAND [ARGS]...",
            "Commands:",
            "benchmark  Run a 6809 Emulation benchmark",
            "tests      Run unittests",
        ], cli_out)

        errors = ["Error", "Traceback"]
        self.assertNotInMultiline(errors, cli_out)
        self.assertNotInMultiline(errors, cli_err)

    def test_benchmark_help(self):
        cli_out, cli_err = self._get("benchmark", "--help")
        #        print(cli_out)
        #        print(cli_err)
        self.assertInMultiline([
            "Usage: cli.py benchmark [OPTIONS]",
            "Run a 6809 Emulation benchmark",
        ], cli_out)

        errors = ["Error", "Traceback"]
        self.assertNotInMultiline(errors, cli_out)
        self.assertNotInMultiline(errors, cli_err)

    def test_run_benchmark(self):
        cli_out, cli_err = self._get("benchmark", "--loops", "1", "--multiply", "1")
        self.assertInMultiline([
            "CRC16 benchmark",
            "Start 1 CRC16 loops",
            "CRC32 benchmark",
            "Start 1 CRC32 loops",
        ], cli_out)

        errors = ["Error", "Traceback"]
        self.assertNotInMultiline(errors, cli_out)
        self.assertNotInMultiline(errors, cli_err)


if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python
# encoding:utf8

"""
    MC6809 - 6809 CPU emulator in Python
    =======================================

    :created: 2014 by Jens Diemer - www.jensdiemer.de
    :copyleft: 2014-2015 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import sys
import locale
import string
import time
import logging

from MC6809.tests.test_6809_program import Test6809_Program, \
    Test6809_Program_Division2
from MC6809.utils.humanize import locale_format_number

PY2 = sys.version_info[0] == 2
if PY2:
    range = xrange


log = logging.getLogger("MC6809")


class Test6809_Program2(Test6809_Program):
    def runTest(self):
        pass

    def bench(self, loops, multiply, func, msg):
        print("\n%s benchmark" % msg)

        self.setUp()
        self.cpu.cycles = 0

        txt = string.printable

        if not PY2:
            txt = bytes(txt, encoding="UTF-8")

        txt = txt * multiply

        print("\nStart %i %s loops with %i Bytes test string..." % (
            loops, msg, len(txt)
        ))

        start_time = time.time()
        for __ in range(loops):
            self._crc32(txt)
        duration = time.time() - start_time

        print("%s benchmark runs %s CPU cycles in %.2f sec" % (
            msg, locale_format_number(self.cpu.cycles), duration
        ))

        return duration, self.cpu.cycles

    def crc32_benchmark(self, loops, multiply):
        return self.bench(loops, multiply, self._crc32, "CRC32")

    def crc16_benchmark(self, loops, multiply):
        return self.bench(loops, multiply, self._crc16, "CRC16")



def run_benchmark(loops, multiply):
    total_duration = 0
    total_cycles = 0
    bench_class = Test6809_Program2()

    #--------------------------------------------------------------------------

    duration, cycles = bench_class.crc16_benchmark(loops, multiply)
    total_duration += duration
    total_cycles += cycles

    #--------------------------------------------------------------------------

    duration, cycles = bench_class.crc32_benchmark(loops, multiply)
    total_duration += duration
    total_cycles += cycles

    #--------------------------------------------------------------------------
    print("-"*79)
    print("\nTotal of %i benchmak loops run in %.2f sec %s CPU cycles." % (
        loops, total_duration, locale_format_number(total_cycles)
    ))
    print("\tavg.: %s CPU cycles/sec" % locale_format_number(total_cycles / total_duration))


if __name__ == '__main__':
    from MC6809.utils.logging_utils import setup_logging

    setup_logging(log,
#        level=1 # hardcore debug ;)
#        level=10 # DEBUG
#        level=20 # INFO
#        level=30 # WARNING
#         level=40 # ERROR
        level=50 # CRITICAL/FATAL
    )

    # will be done in CLI:
    locale.setlocale(locale.LC_ALL, '') # For Formating cycles/sec number

    run_benchmark(
        loops=1
#        loops=2
#        loops=10
    )
    print(" --- END --- ")



#!/usr/bin/env python

"""
    6809 unittests
    ~~~~~~~~~~~~~~

    Register changed Ops: TFR, EXG

    :created: 2014 by Jens Diemer - www.jensdiemer.de
    :copyleft: 2014-2015 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import logging
import sys
import unittest

from MC6809.tests.test_base import BaseCPUTestCase


log = logging.getLogger("MC6809")


class Test6809_TFR(BaseCPUTestCase):
    def test_TFR_A_B(self):
        self.cpu.accu_a.set(0x12)
        self.cpu.accu_b.set(0x34)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1F, 0x89, # TFR A,B
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0x12)
        self.assertEqualHexByte(self.cpu.accu_b.get(), 0x12)

    def test_TFR_B_A(self):
        self.cpu.accu_a.set(0x12)
        self.cpu.accu_b.set(0x34)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1F, 0x98, # TFR B,A
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0x34)
        self.assertEqualHexByte(self.cpu.accu_b.get(), 0x34)

    def test_TFR_X_U(self):
        """
        LDX #$1234
        LDU #$abcd
        TFR X,U
        STX $20
        STU $30
        """
        self.cpu.index_x.set(0x1234)
        self.cpu.user_stack_pointer.set(0xabcd)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1F, 0x13, # TFR X,U
        ])
        self.assertEqualHexWord(self.cpu.index_x.get(), 0x1234)
        self.assertEqualHexWord(self.cpu.user_stack_pointer.get(), 0x1234)

    def test_TFR_CC_X(self):
        """
        transfer 8 bit register in a 16 bit register
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.cc.set(0x34)
        self.cpu_test_run(start=0x4000, end=None, mem=[
            0x1F, 0xA1, # TFR CC,X
        ])
        self.assertEqualHexByte(self.cpu.cc.get(), 0x34)
        self.assertEqualHexWord(self.cpu.index_x.get(), 0xff34)

    def test_TFR_CC_A(self):
        """
        transfer 8 bit register in a 16 bit register
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.accu_a.set(0xab)
        self.cpu.cc.set(0x89)
        self.cpu_test_run(start=0x4000, end=None, mem=[
            0x1F, 0xA8, # TFR CC,A
        ])
        self.assertEqualHexByte(self.cpu.cc.get(), 0x89)
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0x89)

    def test_TFR_Y_B(self):
        """
        transfer 16 bit register in a 8 bit register
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.index_y.set(0x1234)
        self.cpu.accu_b.set(0xab)
        self.cpu_test_run(start=0x4000, end=None, mem=[
            0x1F, 0x29, # TFR Y,B
        ])
        self.assertEqualHexWord(self.cpu.index_y.get(), 0x1234)
        self.assertEqualHexByte(self.cpu.accu_b.get(), 0x34)

    def test_TFR_undefined_A(self):
        """
        transfer undefined register in a 8 bit register
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.accu_a.set(0x12)
        self.cpu_test_run(start=0x4000, end=None, mem=[
            0x1F, 0x68, # TFR undefined,A
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0xff)


class Test6809_EXG(BaseCPUTestCase):
    def test_EXG_A_B(self):
        self.cpu.accu_a.set(0xab)
        self.cpu.accu_b.set(0x12)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1E, 0x89, # EXG A,B
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0x12)
        self.assertEqualHexByte(self.cpu.accu_b.get(), 0xab)

    def test_EXG_X_Y(self):
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x8E, 0xAB, 0xCD, #       LDX   #$abcd     ; set X to $abcd
            0x10, 0x8E, 0x80, 0x4F, # LDY   #$804f     ; set Y to $804f
            0x1E, 0x12, #             EXG   X,Y        ; y,x=x,y
            0x9F, 0x20, #             STX   $20        ; store X to $20
            0x10, 0x9F, 0x40, #       STY   $40        ; store Y to $40
        ])
        self.assertEqualHexWord(self.cpu.memory.read_word(0x20), 0x804f) # X
        self.assertEqualHexWord(self.cpu.memory.read_word(0x40), 0xabcd) # Y

    def test_EXG_A_X(self):
        """
        exange 8 bit register with a 16 bit register
        TODO: verify this behaviour on real hardware!
        """
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x86, 0x56, #             LDA   #$56
            0x8E, 0x12, 0x34, #       LDX   #$1234
            0x1E, 0x81, #             EXG   A,X
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0x34)
        self.assertEqualHexWord(self.cpu.index_x.get(), 0xff56)

    def test_EXG_A_CC(self):
        """
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.accu_a.set(0x1f)
        self.cpu.cc.set(0xe2)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1E, 0x8A, # EXG A,CC
        ])
        self.assertEqualHexByte(self.cpu.accu_a.get(), 0xe2)
        self.assertEqualHexByte(self.cpu.cc.get(), 0x1f)

    def test_EXG_X_CC(self):
        """
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.index_x.set(0x1234)
        self.cpu.cc.set(0x56)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1E, 0x1A, # EXG X,CC
        ])
        self.assertEqualHexWord(self.cpu.index_x.get(), 0xff56)
        self.assertEqualHexByte(self.cpu.cc.get(), 0x34)

    def test_EXG_undefined_to_X(self):
        """
        TODO: verify this behaviour on real hardware!
        """
        self.cpu.index_x.set(0x1234)
        self.cpu_test_run(start=0x2000, end=None, mem=[
            0x1E, 0xd1, # EXG undefined,X
        ])
        self.assertEqualHexWord(self.cpu.index_x.get(), 0xffff)



if __name__ == '__main__':
    unittest.main(
        argv=(
            sys.argv[0],
#            "Test6809_TFR",
#             "Test6809_TFR.test_TFR_CC_A",
#            "Test6809_EXG",
        ),
#         verbosity=1,
        verbosity=2,
#         failfast=True,
    )



#!/usr/bin/env python

"""
    MC6809 - CLI
    ~~~~~~~~~~~~

    :created: 2015 by Jens Diemer - www.jensdiemer.de
    :copyleft: 2015 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


import sys
import unittest

try:
    import click
except ImportError as err:
    print("Import error: %s" % err)
    print()
    print("Please install 'click' !")
    print("more info: http://click.pocoo.org")
    sys.exit(-1)

from MC6809.tests import run_tests
from MC6809.core.bechmark import run_benchmark


@click.group()
def cli():
    pass


DEFAULT_LOOPS = 5
DEFAULT_MULTIPLY = 15
@cli.command(help="Run a 6809 Emulation benchmark")
@click.option("--loops", default=DEFAULT_LOOPS,
    help="How many benchmark loops should be run? (default: %i)" % DEFAULT_LOOPS)
@click.option("--multiply", default=DEFAULT_MULTIPLY,
    help="Test data multiplier (default: %i)" % DEFAULT_MULTIPLY)
def benchmark(loops, multiply):
    run_benchmark(loops, multiply)


@cli.command(help="Run unittests")
@click.option('--verbosity', default=2, help='Number for verbosity settings')
@click.option('--failfast', default=False, help='Number for verbosity settings', is_flag=True)
def tests(verbosity, failfast):
    run_tests(verbosity, failfast)


if __name__ == "__main__":
    cli()


#!/usr/bin/env python
# coding: utf-8

"""
    distutils setup
    ~~~~~~~~~~~~~~~

    :copyleft: 2014-2015 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

from setuptools import setup, find_packages
import os
import sys
import subprocess
import shutil

from MC6809 import __version__


PACKAGE_ROOT = os.path.dirname(os.path.abspath(__file__))


# convert creole to ReSt on-the-fly, see also:
# https://code.google.com/p/python-creole/wiki/UseInSetup
try:
    from creole.setup_utils import get_long_description
except ImportError as err:
    if "check" in sys.argv or "register" in sys.argv or "sdist" in sys.argv or "--long-description" in sys.argv:
        raise ImportError("%s - Please install python-creole >= v0.8 -  e.g.: pip install python-creole" % err)
    long_description = None
else:
    long_description = get_long_description(PACKAGE_ROOT)


if "test" in sys.argv or "nosetests" in sys.argv:
    """
    nose is a optional dependency, so test import.
    Otherwise the user get only the error:
        error: invalid command 'nosetests'
    """
    try:
        import nose
    except ImportError as err:
        print("\nError: Can't import 'nose': %s" % err)
        print("\nMaybe 'nose' is not installed or virtualenv not activated?!?")
        print("e.g.:")
        print("    ~/your/env/$ source bin/activate")
        print("    ~/your/env/$ pip install nose")
        print("    ~/your/env/$ ./setup.py nosetests\n")
        sys.exit(-1)
    else:
        if "test" in sys.argv:
            print("\nPlease use 'nosetests' instead of 'test' to cover all tests!\n")
            print("e.g.:")
            print("     $ ./setup.py nosetests\n")
            sys.exit(-1)


if "publish" in sys.argv:
    """
    'publish' helper for setup.py

    Build and upload to PyPi, if...
        ... __version__ doesn't contains "dev"
        ... we are on git 'master' branch
        ... git repository is 'clean' (no changed files)

    Upload with "twine", git tag the current version and git push --tag

    The cli arguments will be pass to 'twine'. So this is possible:
     * Display 'twine' help page...: ./setup.py publish --help
     * use testpypi................: ./setup.py publish --repository=test

    TODO: Look at: https://github.com/zestsoftware/zest.releaser

    Source: https://github.com/jedie/python-code-snippets/blob/master/CodeSnippets/setup_publish.py
    copyleft 2015 Jens Diemer - GNU GPL v2+
    """
    if sys.version_info[0] == 2:
        input = raw_input

    try:
        # Test if wheel is installed, otherwise the user will only see:
        #   error: invalid command 'bdist_wheel'
        import wheel
    except ImportError as err:
        print("\nError: %s" % err)
        print("\nMaybe https://pypi.python.org/pypi/wheel is not installed or virtualenv not activated?!?")
        print("e.g.:")
        print("    ~/your/env/$ source bin/activate")
        print("    ~/your/env/$ pip install wheel")
        sys.exit(-1)

    try:
        import twine
    except ImportError as err:
        print("\nError: %s" % err)
        print("\nMaybe https://pypi.python.org/pypi/twine is not installed or virtualenv not activated?!?")
        print("e.g.:")
        print("    ~/your/env/$ source bin/activate")
        print("    ~/your/env/$ pip install twine")
        sys.exit(-1)

    def verbose_check_output(*args):
        """ 'verbose' version of subprocess.check_output() """
        call_info = "Call: %r" % " ".join(args)
        try:
            output = subprocess.check_output(args, universal_newlines=True, stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as err:
            print("\n***ERROR:")
            print(err.output)
            raise
        return call_info, output

    def verbose_check_call(*args):
        """ 'verbose' version of subprocess.check_call() """
        print("\tCall: %r\n" % " ".join(args))
        subprocess.check_call(args, universal_newlines=True)

    if "dev" in __version__:
        print("\nERROR: Version contains 'dev': v%s\n" % __version__)
        sys.exit(-1)

    print("\nCheck if we are on 'master' branch:")
    call_info, output = verbose_check_output("git", "branch", "--no-color")
    print("\t%s" % call_info)
    if "* master" in output:
        print("OK")
    else:
        print("\nNOTE: It seems you are not on 'master':")
        print(output)
        if input("\nPublish anyhow? (Y/N)").lower() not in ("y", "j"):
            print("Bye.")
            sys.exit(-1)

    print("\ncheck if if git repro is clean:")
    call_info, output = verbose_check_output("git", "status", "--porcelain")
    print("\t%s" % call_info)
    if output == "":
        print("OK")
    else:
        print("\n *** ERROR: git repro not clean:")
        print(output)
        sys.exit(-1)

    print("\ncheck if pull is needed")
    verbose_check_call("git", "fetch", "--all")
    call_info, output = verbose_check_output("git", "log", "HEAD..origin/master", "--oneline")
    print("\t%s" % call_info)
    if output == "":
        print("OK")
    else:
        print("\n *** ERROR: git repro is not up-to-date:")
        print(output)
        sys.exit(-1)
    verbose_check_call("git", "push")

    print("\nCleanup old builds:")
    def rmtree(path):
        path = os.path.abspath(path)
        if os.path.isdir(path):
            print("\tremove tree:", path)
            shutil.rmtree(path)
    rmtree("./dist")
    rmtree("./build")

    print("\nbuild but don't upload...")
    log_filename="build.log"
    with open(log_filename, "a") as log:
        call_info, output = verbose_check_output(
            sys.executable or "python",
            "setup.py", "sdist", "bdist_wheel", "bdist_egg"
        )
        print("\t%s" % call_info)
        log.write(call_info)
        log.write(output)
    print("Build output is in log file: %r" % log_filename)

    print("\ngit tag version (will raise a error of tag already exists)")
    verbose_check_call("git", "tag", "v%s" % __version__)

    print("\nUpload with twine:")
    twine_args = sys.argv[1:]
    twine_args.remove("publish")
    twine_args.insert(1, "dist/*")
    print("\ttwine upload command args: %r" % " ".join(twine_args))
    from twine.commands.upload import main as twine_upload
    twine_upload(twine_args)

    print("\ngit push tag to server")
    verbose_check_call("git", "push", "--tags")

    sys.exit(0)




setup(
    name="MC6809",
    version=__version__,
    py_modules=["MC6809"],
    provides=["MC6809"],
    install_requires=[
        "click",
    ],
    tests_require=[
        "nose", # https://pypi.python.org/pypi/nose
    ],
    entry_points='''
        [console_scripts]
        MC6809=MC6809.cli:cli
    ''',
    author="Jens Diemer",
    author_email="MC6809@jensdiemer.de",
    description="MC6809 CPU emulator written in Python",
    keywords="Emulator 6809",
    long_description=long_description,
    url="https://github.com/6809/MC6809",
    license="GPL v3+",
    classifiers=[
        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: System :: Emulators",
        "Topic :: Software Development :: Assemblers",
        "Topic :: Software Development :: Testing",
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)



#!/usr/bin/env python
# coding: utf-8

"""
    DragonPy - Humanize
    ===================

    :copyleft: 2013-2014 by the MC6809 team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import locale
import sys
import platform


def locale_format_number(val):
    """
    Depend on users local, so no active doctest here ;)

    > locale_format_number(1234567.89)
    '1.234.567.890'
    """
    try:
        return locale.format('%d', val, 1)
    except UnicodeDecodeError:
        # For PyPy3, see: https://bitbucket.org/pypy/pypy/issue/1858/pypy3-localeformat-d-val-1
#        return '{:n}'.format(val) # makes 1234567890.1234 to 1,23457e+09 :(
        return '{:,}'.format(int(val))


def byte2bit_string(data):
    """
    >>> byte2bit_string(0x1b)
    '00011011'
    """
    return '{0:08b}'.format(data)


def nice_hex(v):
    """
    >>> nice_hex(0x1)
    '$01'
    >>> nice_hex(0x123)
    '$0123'
    """
    if v < 0x100:
        return "$%02x" % v
    if v < 0x10000:
        return "$%04x" % v
    return "$%x" % v


def hex_repr(d):
    """
    >>> hex_repr({"A":0x1,"B":0xabc})
    'A=$01 B=$0abc'
    """
    txt = []
    for k, v in sorted(d.items()):
        if isinstance(v, int):
            txt.append("%s=%s" % (k, nice_hex(v)))
        else:
            txt.append("%s=%s" % (k, v))
    return " ".join(txt)


def cc_value2txt(status):
    """
    >>> cc_value2txt(0x50)
    '.F.I....'
    >>> cc_value2txt(0x54)
    '.F.I.Z..'
    >>> cc_value2txt(0x59)
    '.F.IN..C'
    """
    return "".join([
        "." if status & x == 0 else char
        for char, x in zip("EFHINZVC", (128, 64, 32, 16, 8, 4, 2, 1))
    ])


def get_python_info():
    implementation = platform.python_implementation()
    if implementation == "CPython":
        return "%s v%s [%s]" % (
            implementation,
            platform.python_version(),
            platform.python_compiler(),
        )
        # e.g.: CPython v2.7.6 [GCC 4.8.2]
    elif implementation == "PyPy":
        ver_info = sys.version.split("(", 1)[0]
        ver_info += sys.version.split("\n")[-1]
        return "Python %s" % ver_info
        # e.g.: Python 2.7.6 [PyPy 2.3.1 with GCC 4.8.2]
    else:
        return "%s %s" % (
            sys.executable,
            sys.version.replace("\n", " ")
        )


if __name__ == "__main__":
    import doctest
    print(doctest.testmod(verbose=0))
